# -*- coding: utf-8 -*-
import datetime
import os

from django.db import models
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User
from django.conf import settings
from django.db.models import Q
from django.template.defaultfilters import slugify

#from sorl.thumbnail.fields import ImageField
from jumbo.middleware.thread_local import get_request
from jumbo.db.models import DateModel, UserModel, OrderedModel, StatusModel, HTMLField
from jumbo.apps.i18n.models import TranslationModel, I18NModel
from jumbo.middleware.thread_local import get_current_user
from jumbo.utils.functions import get_settings

from jcmsseo.models import SeoData


##############################
# TIPO NEWS                  #
##############################
class CategoryManager(models.Manager):
    def default(self):
        return self.get_query_set().get(default=True)

class Category(DateModel, UserModel, StatusModel, OrderedModel):
    objects = CategoryManager()

    ## campi modello
    name = models.CharField(
        unique=True,
        max_length=25,
        verbose_name=_('News type'),
    )
    default = models.BooleanField(
        default=False,
        verbose_name=_('Default category'),
        help_text=_('Is the default one?'),
    )

    def __unicode__(self):
        return u"%s" % self.name

    class Meta:
        verbose_name = _('News type')
        verbose_name_plural = _('News types')

    def save(self, force_insert=False, force_update=False):
        super(Category, self).save(force_insert=force_insert, force_update=force_update)
        if self.default:
            for sc in Category.objects.exclude(pk=self.id):
                sc.default = False
                sc.save(force_insert=force_insert, force_update=force_update)
        try:
            sc = Category.objects.get(default=True)
        except models.ObjectDoesNotExist:
            self.default = True
            super(Category, self).save(force_insert=force_insert, force_update=force_update)


##########################
# NEWS                   #
##########################

def get_news_upload_to(instance, filename):
    try:
        filename, estensione = filename.split('.')
    except:
        estensione = "" #filename senza punto
    return "%s/%s/%s/%s.%s" % (get_settings("news.settings", "NEWS_PATH"),
                            instance.event_date,
                            slugify(instance.name)[0:16],
                            slugify(filename), estensione)

class PermittedNewsManager(models.Manager):
    def get_query_set(self):
        user = get_current_user()

        qs = super(PermittedNewsManager, self).get_query_set()

        if user.is_superuser:
            return qs

        if not user.is_authenticated():
            return qs.filter(users=None)
        else:
            q1 = Q(users = None)
            q2 = Q(users = user)
            return qs.filter(q1 | q2)

class ActiveNewsManager(PermittedNewsManager):
    def get_query_set(self):
        now = datetime.datetime.now()
        q1 = Q(pub_date_begin__lte=now) | Q(pub_date_begin__isnull=True)
        q2 = Q(pub_date_end__gte=now) | Q(pub_date_end__isnull=True)
        return super(ActiveNewsManager, self).get_query_set().filter(status=1).filter(q1 & q2)

class News(DateModel, UserModel, OrderedModel, StatusModel, I18NModel, SeoData):
    ## ForeignKey
    categories = models.ManyToManyField(
        Category,
        null=True,
        blank=True,
        related_name = "news",
        verbose_name = _('Categories')
    )
    users = models.ManyToManyField(
        User,
        blank=True,
        null=True,
        related_name='news',
        verbose_name=_('Visible only to'),
        help_text=_('News visible to these users, if empty is visible to all users.'),
    )
    ## Campi
    name = models.CharField(
        unique=True,
        max_length=100,
        verbose_name=_('News'),
    )
    event_date = models.DateField(
        blank=True,
        null=True,
        verbose_name=_('News date'),
        help_text=_('Date which news refers to.'),
    )
    pub_date_begin = models.DateTimeField(
        blank=True,
        null=True,
        verbose_name=_('Publication date begin'),
        help_text=_('When news publication date begins.'),
    )
    pub_date_end = models.DateTimeField(
        blank=True,
        null=True,
        verbose_name=_('Publication date end'),
        help_text=_('When news publication date ends.'),
    )
    title = models.CharField(
        max_length=255,
        blank=True,
        null=True,
        verbose_name=_('Default title'),
    )
    small_image = models.ImageField(
        upload_to='news/thumbnail',
        blank = True, null = True,
        verbose_name = _("Small Image")
    )
    big_image = models.ImageField(
        upload_to='news/bigimage',
        blank = True, null = True,
        verbose_name = _("Big Image")
    )
    short_description = models.CharField(
        max_length=100,
        verbose_name=_('Short Description'),
        blank = True, null = True
    )
    text = HTMLField(
        blank=True,
        null=True,
        verbose_name=_('Long Description'),
    )
    note = models.TextField(
        blank=True,
        verbose_name=_('Note'),
    )
    file = models.FileField(
        upload_to=get_news_upload_to,
        verbose_name = _("File"),
        blank=True,
        null=True,
    )
    file_due = models.FileField(
        upload_to=get_news_upload_to,
        verbose_name = _("Altro File"),
        blank=True,
        null=True,
    )
    link = models.URLField(
        blank = True, null = True,
        verify_exists=False,
        help_text = _("If not blank, override the default's news url")
    ) # se inserito forza il link della news a questo
    force_in_homepage = models.BooleanField(default = False)

    objects = PermittedNewsManager() # The default manager.
    active = ActiveNewsManager()
    
    @staticmethod
    def get_anni(categorie=None):
        output = []
        for n in News.active.values_list('event_date').order_by('event_date'):
            if n[0]:
                if (n[0].year, n[0].year) not in output:
                    output.append((n[0].year, n[0].year))
        output.reverse()
        return output
        
    @staticmethod
    def get_mesi(anno=None, categorie=None):
        return (("1", "Gennaio"), ("2", "Febbraio"), ("3", "Marzo"),
                ("4", "Aprile"), ("5", "Maggio"), ("6", "Giugno"),
                ("7", "Luglio"), ("8", "Agosto"), ("9", "Settembre"),
                ("10", "Ottobre"), ("11", "Novembre"), ("12", "Dicembre"))
    
    def save(self, *args, **kwargs):
        if not self.pk and not self.title:
            self.title = self.name
        super(News, self).save(*args, **kwargs)


    def get_users(self):
        users = self.users.all().order_by('username')
        if users:
            return u', '.join([user.username for user in users])
        else:
            return _(u'All')
    get_users.short_description = _('show to')


    def get_categories(self):
        categories = self.categories.all().order_by('name')
        if categories:
            return u', '.join([category.name for category in categories])
        else:
            return _(u'All')
    get_categories.short_description = _('Categories')


    def get_short_description(self):
        return self.short_description or "%s ....." % self.text[:100]

    def get_filename_uno(self):
        return os.path.basename(self.file.name)

    def get_filename_due(self):
        return os.path.basename(self.file_due.name)

    class Meta:
        verbose_name = _("News")
        verbose_name_plural = _("News")
        ordering = ("event_date",)

    def __unicode__(self):
        if self.title:
            return self.title
        return self.name

##########################
# NEWS TRANSLATION       #
##########################
class NewsTranslation(DateModel, UserModel, StatusModel, TranslationModel):
    ## ForeignKey
    news = models.ForeignKey(
        News,
        related_name='translations',
        verbose_name=_('News type'),
    )
    title = models.CharField(
        max_length=256,
        verbose_name=_('Title'),
    )
    text = HTMLField(
        verbose_name=_('Body'),
    )

    class Meta:
        verbose_name = _("News translation")
        verbose_name_plural = _("News translations")
        unique_together = (('news','language',),)

    def __unicode__(self):
        return u'%s (%s)' % (self.news.title, self.language)

